%code requires {
  #include "te_node.h"
  #include <string.h>
  #include <stdio.h>
  #include <stdlib.h>
}

%code {
  extern int yylex();
  extern int yyerror();

  typedef struct var_info{
    char *name;
    te_node *type;
    struct var_info *next;
  }var_info;

  var_info *var_table = NULL;

  var_info *var_table_get(char *name){
    var_info *v;
    for(v = var_table; v != NULL; v = v->next){
      if(!strcmp(name, v->name))
        return v;
    }
    return NULL;
  }

  int var_table_put(char *name, te_node *type){
    if(var_table_get(name) != NULL)
      return 1;
    var_info *new_node = (var_info *) malloc(sizeof(var_info));
    new_node->name = name;
    new_node->type = type;
    new_node->next = var_table;
    var_table = new_node;
    return 0;
  }

  void free_var_table(){
    var_info *v;
    for(v = var_table; v != NULL; v = v->next){
      free((void *)v->name);
      te_free(v->type);
    }
  }

  int add_var(char *name, te_node *type){
    printf("var %s:", name);
    te_print(type);
    printf("\n");
    return var_table_put(name, type);
  }

  type_entry *type_table = NULL;

  type_entry *type_table_get(char *name){
    type_entry *t;
    for(t = type_table; t != NULL; t = t->next){
      if(!strcmp(name, t->name))
        return t;
    }
    return NULL;
  }

  int type_table_put(char *name, te_node *type){
    if(type_table_get(name) != NULL)
      return 1;
    type_entry *new_node = (type_entry *) malloc(sizeof(type_entry));
    new_node->name = name;
    new_node->type = type;
    new_node->next = type_table;
    type_table = new_node;
    return 0;
  }

  void free_type_table(){
    type_entry *t;
    for(t = type_table; t != NULL; t = t->next){
      free((void *)t->type);
    }
  }
}

%code provides{
  typedef struct type_entry{
      char *name;
      te_node *type;
      struct type_entry *next;
    }type_entry;
}

%debug

%union {
  int int_value;
  char *string;
  te_node *te;
}

%token <int_value> NUM;
%token <string> ID;
%token <te> TYPE;
%token TIMES SO SC S CM;

%type <te> Decll Decl T Vlist V Ptr Ary NT0;   

%%

Decll : /* empty */ {;}
          | Decll Decl S
	  | Decll error S { printf("Error in declaration\n"); }
;


Decl : T Vlist
;

//BASE TYPE: int, float, char, double
T : TYPE
	{ 
		$$ = $1; 
	}
;


NT0 : /* Empty */{
	$$ = $<te>-1;
};

Vlist : V { $$ = $1; }
	| Vlist CM NT0 V { $$=$1;	}
;

//ARRAYS, POINTERS AND IDENTIFIER


V :	Ptr ID Ary 
	{
			add_var($2, $3); 
			$$ = $<te>0;
	}
;


//POINTERS
Ptr :	/* empty */ 
	{
		$$ = $<te>0;
	}
	| Ptr TIMES 
	{
		$$ = te_make_pointer($1); 
	}
;

//ARRAYS
Ary : /* empty */ 
	{ 
		$$ = $<te>-1;
	}
	| Ary SO NUM SC 
	{
		$$ = te_make_array($3, $1); 
	}
;