#include "te_node.h"
#include <stdlib.h>
#include <stdio.h>

te_node *te_make_base(int code){
    te_node *p = (te_node *) malloc(sizeof(te_node));
    p->tag = TEBASETYPE;
    p->code = code;
    return p;
}
te_node *te_make_pointer(te_node *base){
    te_node *p = (te_node *) malloc(sizeof(te_node));
    p->tag = TEPOINTER;
    p->left = base;
    return p;
}
te_node *te_make_array(int size, te_node *base){
    te_node *p = (te_node *) malloc(sizeof(te_node));
    p->tag = TEARRAY;
    p->left = base;
    p->size = size;
    return p;
}
/* Uncomment for struct and typedef 
te_node *te_make_name(char *name){
    te_node *p = (te_node *) malloc(sizeof(te_node));
    p->tag = TENAME;
    p->name = name;
    return p;
}
te_node *te_make_product(te_node *left, te_node *right){
    te_node *p = (te_node *) malloc(sizeof(te_node));
    p->tag = TEPRODUCT;
    p->left = left;
    p->right = right;
    return p;
}
te_node *te_make_fwdstruct(char *name){
    te_node *p = te_make_struct(NULL, name);
    p->tag = TEFWDSTRUCT;
    return p;
}
int te_is_fwd(te_node *str){
    return str->tag == TEFWDSTRUCT;
}
void te_cons_struct(te_node *str, te_node *fields){
    str->tag = TESTRUCT;
    str->left = fields;
}
te_node *te_make_struct(te_node *fields, char *name){
    te_node *p = (te_node *) malloc(sizeof(te_node));
    p->tag = TESTRUCT;
    p->left = fields;
    p->name = name;
    return p;    
}
te_node *te_make_function(te_node *domain, te_node *range){
    te_node *p = (te_node *) malloc(sizeof(te_node));
    p->tag = TEFUNCTION;
    p->left = domain;
    p->right = range;
    return p;
}
*/
void te_print(te_node *node){
    int i;
    switch (node->tag){
    case TEBASETYPE:
        switch (node->code)
        {
        case 1:
            printf("[INT]");
            break;
        case 2:
            printf("[FLOAT]");
            break;
        case 3:
            printf("[CHAR]");
            break;
        case 4:
            printf("[DOUBLE]");
            break;
        default:
            break;
        }
        break;
    case TEARRAY:
        printf("array(%d,", node->size);
        te_print(node->left);
        printf(")");
        break;
    case TEPOINTER :
	    printf("pointer(");
		te_print(node->left);
		printf(")");
		break;
    /* Uncomment for struct and typedef 
	case TENAME :
		printf("\"%s\"", node->name);
		break;
	case TEPRODUCT :
		printf("(");
		te_print(node->left);
		printf(" X ");
		te_print(node->right);
		printf(")");
		break;
	case TESTRUCT :
		i = node->tag; // evita la ricorsione 
		node->tag = -1;
		printf("struct %s(", node.name);
		te_print(node->left);
		printf(")");
		node->tag = i;
		break;
	case TEFUNCTION :
		printf("(");
		te_print(node->left);
		printf(" . ");
		te_print(node->right);
		printf(")");
		break;
	case -1 :
		printf("struct %s", node->name);
		break;
    */
    }
}
void te_free(te_node *node){
    te_node *n = node;
    if(n->tag == TEBASETYPE)
        return;
    if(n->left != NULL)
        te_free(n->left);
    if(n->right != NULL)
        te_free(n->right);
    free((void *)n);
}